% Constant panel method for a 2D Neumann Problem. April 12, 2006. 
% Compute velocity potential, velocities, and pressure on body surface
% by using the source method
% Example: uniform flow past an arbitrary body in an infinite fluid.
% The program is written for arbitrary body geometry and flow speed.
% Results are plotted for a circular cylinder of radius R=2m and 
% uniform flow speed U=1m/s.

% input flow speed and fluid density
u = input(' Enter Flow Speed: ');
rho = 1000; % water density m/s.

% input body geometry and obtain basic panel information 
[npanels,xvert,yvert,xc,yc,len,nx,ny] = setpanels;

% evaluate the polar angle of the centroid of each panel 
%(angle varies from 0 to 2pi)
thc = atan2(yc,xc);
for k=1:npanels;
    if thc(k) < 0;
        thc(k) = thc(k)+2*pi;
    end;
end;

%% build up the system of equation: Ax=b with x being unknown source
%% strength, A and b being influece coefficients

% find G(x,y;xi,zeta),Gx(x,y;xi,zeta), Gy(x,y;xi,zeta), i.e. potential G, 
% x-velocity Gx and y-velocity at centroid of panel (x,y) due to constant 
% source distribuiton on panel (xi,zeta).
[g,gx,gy] =inflcoef(npanels,xvert,yvert,xc,yc,nx,ny,len);

% find normal velocity on body surface
dpotdn = setbcs(u,nx);

% determine influence coefficients A and b
[A,b] = matrices(npanels,nx,ny,gx,gy,dpotdn);

% solve Ax=b for unknown source strength
source_strength = A\b'; 


%% evaluate velocity potential on body surface:
for i=1:npanels
    phi(i)=g(i,:)*source_strength;
end

% compare the numerical solution to the analytic solution for the 
% potential on body surface (for a circular cylinder)
figure(1)
phi_analytic=-u*xc;
plot(thc,phi,'r-',thc,phi_analytic,'g*')
xlabel('angle');
ylabel('velocity potential');
legend('numerical result','analytical result');


% evaluate tangential velocity on the body surface 
% and compare with the analytical solution
for i=1:npanels
% numerical solution    
    vtc(i)=(gx(i,:)*source_strength)*ny(i)+ ...
           (gy(i,:)*source_strength)*(-nx(i))-u*ny(i);
% analytic solution 
       vth(i) = 2.0*u*sin(thc(i));
end

figure(2)
plot(thc, vtc, 'r-', thc, vth, 'g*');
xlabel('angle');
ylabel('tangential velocity');
legend('numerical result','analytical result');

%evaluate the distribution of the presure coefficient on the body
p = -0.5*rho*(vtc .* vtc - u*u);
cp = p/(0.5*rho*u*u);
cp_analytical=1-4*sin(thc).^2;

figure(3)
plot(thc,cp,'r-',thc,cp_analytical,'g*');
xlabel('angle');
ylabel ('Pressure Coefficient (Cp)');       
legend('numerical result','analytical result');